// src/index.tsx
import { List } from "@solid-primitives/list";
import { createDeepSignal } from "@solid-primitives/resource";
import clsx from "clsx";
import { codeToHast, getHighlighter } from "shiki";
import {
  Index,
  Show,
  createEffect,
  createMemo,
  createResource,
  createSignal,
  onCleanup,
  onMount,
  useTransition
} from "solid-js";
import { Dynamic } from "solid-js/web";

// src/index.module.css
var src_default = {};

// src/utils/calculate-contrasting-color.ts
import { colord, extend } from "colord";
import mixPlugin from "colord/plugins/mix";
extend([mixPlugin]);
function calculateContrastingColor(hexColor) {
  const backgroundColor = colord(hexColor);
  let selectionBackgroundColor;
  if (backgroundColor.isLight()) {
    selectionBackgroundColor = backgroundColor.darken(0.1);
  } else {
    selectionBackgroundColor = backgroundColor.lighten(0.15);
  }
  return selectionBackgroundColor.toRgbString();
}

// src/utils/process-props.ts
import { mergeProps, splitProps } from "solid-js";
function processProps(props, defaults, split) {
  return splitProps(defaultProps(props, defaults), split || []);
}
function defaultProps(props, defaults) {
  return mergeProps(defaults, props);
}

// src/index.tsx
var calculateMaxCharCount = (source) => {
  let maximumLineSize = -Infinity;
  source.split("\n").forEach((line) => {
    if (line.length > maximumLineSize) {
      maximumLineSize = line.length;
    }
  });
  return maximumLineSize;
};
function ShikiTextarea(props) {
  const [config, rest] = processProps(props, { lang: "tsx", theme: "min-light" }, [
    "class",
    "defaultValue",
    "lang",
    "onInput",
    "value",
    "style",
    "theme"
  ]);
  const [source, setSource] = createSignal(config.defaultValue || config.value);
  const [characterDimensions, setCharacterDimensions] = createSignal({
    width: 0,
    height: 0
  });
  const maxCharCount = createMemo(() => calculateMaxCharCount(source()));
  const lineCount = createMemo(() => source().split("\n").length);
  const [, startTransition] = useTransition();
  const [themeStyles] = createResource(
    () => [config.theme, config.lang],
    ([theme, lang]) => getHighlighter({ themes: [theme], langs: [lang] }).then((highlighter) => highlighter.getTheme(theme)).then((theme2) => ({
      "--theme-selection-color": calculateContrastingColor(theme2.bg),
      "--theme-background-color": theme2.bg,
      "--theme-foreground-color": theme2.fg
    }))
  );
  createEffect(() => setSource(config.value));
  return <div
    class={clsx(src_default.editor, config.class)}
    style={{
      ...themeStyles(),
      ...config.style,
      "--line-count": lineCount(),
      "--max-char-count": maxCharCount(),
      "--char-width": characterDimensions().width,
      "--char-height": characterDimensions().height
    }}
    {...rest}
  ><div class={src_default.container}><List each={source().split("\n")}>{(line) => <Line source={line()} lang={config.lang} theme={config.theme} />}</List><textarea
    inputmode="none"
    autocomplete="off"
    spellcheck={false}
    class={src_default.textarea}
    onInput={({ currentTarget: { value } }) => {
      startTransition(() => {
        setSource(value);
        config.onInput?.(value);
      });
    }}
    value={config.value}
  /><CharacterDimensions onResize={setCharacterDimensions} /></div></div>;
}
function Line(props) {
  const [hast] = createResource(
    () => [props.source, props.theme, props.lang],
    ([source, theme, lang]) => source ? codeToHast(source, { lang, theme }).then(({ children }) => children[0]) : { children: void 0 },
    { storage: createDeepSignal }
  );
  const hastNodes = () => (hast() || hast.latest)?.children;
  return <div class={src_default.line}><List each={hastNodes()}>{(child) => <HastNode node={child()} />}</List></div>;
}
function HastNode(props) {
  return <Show when={props.node.type !== "text" && props.node} fallback={props.node.value}>{(node) => <Dynamic component={node().tagName || "div"} {...node().properties}><Index each={node().children}>{(child) => <HastNode node={child()} />}</Index></Dynamic>}</Show>;
}
function CharacterDimensions(props) {
  const [character, setCharacter] = createSignal(null);
  onMount(() => {
    const resizeObserver = new ResizeObserver(() => {
      const { width, height } = character().getBoundingClientRect();
      props.onResize({ width, height });
    });
    resizeObserver.observe(character());
    onCleanup(() => resizeObserver.disconnect());
  });
  return <code ref={setCharacter} class={src_default.character} innerText="m" aria-hidden />;
}
export {
  ShikiTextarea
};
